/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.dbl;

// BRIDGE API
import com.rempl.readers.its.bridge.Bridge;
import com.rempl.readers.its.bridge.BridgeException;
import com.rempl.readers.its.bridge.Issue;

// autogenerated JAXB classes
import com.rempl.readers.its.dbl.jaxb.Issues;
import com.rempl.readers.its.dbl.jaxb.XmlIssue;

// file manipulation
import java.io.File;

// collection of issues
import java.util.HashMap;
import java.util.Map;

// JAXB
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;

/**
 * Mocked/stubbed version of a bridge.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: BridgeDbl.java 1306 2011-05-02 16:04:25Z guard $
 */
public final class BridgeDbl implements Bridge {

    /**
     * XML location config param.
     */
    private static final String KEY_XML = "issues";

    /**
     * Collection of issues.
     */
    private Map<String, Issue> issues;

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(final Map<String, String> params)
        throws BridgeException {
        if (!params.containsKey(this.KEY_XML)) {
            throw new BridgeException(
                "Configuration parameter \""
                + this.KEY_XML
                + "\" is mandatory"
            );
        }
        final File xml = new File(params.get(this.KEY_XML));
        this.issues = this.issues(xml);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Issue> retrieve() throws BridgeException {
        return this.issues;
    }

    /**
     * Unmarshal from XML into Java objects and return
     * the root object.
     * @param xml The XML file to read from
     * @return The map of issues
     * @throws BridgeException If something goes wrong
     * @see #configure(Map)
     */
    private Map<String, Issue> issues(final File xml)
        throws BridgeException {
        final Map<String, Issue> map = new HashMap<String, Issue>();
        final Issues jaxbIssues = this.unmarshal(xml);
        for (XmlIssue issue : jaxbIssues.getIssue()) {
            map.put(issue.getName(), new IssueDbl(issue));
        }
        return map;
    }

    /**
     * Unmarshal from XML into Java objects and return
     * the root object.
     * @param xml The XML file to read from
     * @return The list of issues
     * @throws BridgeException If something goes wrong
     */
    private Issues unmarshal(final File xml)
        throws BridgeException {
        Object jaxbIssues;
        try {
            final JAXBContext ctx =
                JAXBContext.newInstance("com.rempl.readers.its.dbl.jaxb");
            final Unmarshaller unmrsh = ctx.createUnmarshaller();
            jaxbIssues = unmrsh.unmarshal(xml);
        } catch (javax.xml.bind.JAXBException ex) {
            throw new BridgeException(ex);
        }
        return (Issues) jaxbIssues;
    }

}
